@@ -13,6 +13,7 @@
 #include <functional>
 #include <iterator>
 #include <memory>
+#include <stdexcept>
 
 #include "xstorage.hpp"
 
@@ -320,7 +321,7 @@ namespace xt
             {
                 allocator_type al = std::allocator_traits<allocator_type>::select_on_container_copy_construction(rhs.get_allocator());
                 pointer tmp = safe_init_allocate(al, rhs.m_size);
-                if (std::is_trivially_default_constructible<value_type>::value)
+                if (xtrivially_default_constructible<value_type>::value)
                 {
                     std::uninitialized_copy(*(rhs.p_data), *(rhs.p_data) + rhs.m_size, tmp);
                 }
