@@ -80,7 +80,7 @@ namespace xt
         using const_stepper = typename iterable_base::const_stepper;
 
         static constexpr layout_type static_layout = layout_type::any;
-        static constexpr bool contiguous_layout = true;
+        static constexpr bool contiguous_layout = false;
 
         template <class Func>
         xgenerator(Func&& f, const S& shape) noexcept;
@@ -209,7 +209,7 @@ namespace xt
      * Returns a constant reference to the element at the specified position in the function.
      * @param first iterator starting the sequence of indices
      * @param last iterator ending the sequence of indices
-     * The number of indices in the squence should be equal to or greater
+     * The number of indices in the sequence should be equal to or greater
      * than the number of dimensions of the container.
      */
     template <class F, class R, class S>
