@@ -67,7 +67,6 @@ namespace xt
     class xindexview : public xview_semantic<xindexview<CT, I>>,
                        public xexpression_iterable<xindexview<CT, I>>
     {
-
     public:
 
         using self_type = xindexview<CT, I>;
@@ -139,12 +138,12 @@ namespace xt
         template <class ST>
         stepper stepper_begin(const ST& shape);
         template <class ST>
-        stepper stepper_end(const ST& shape);
+        stepper stepper_end(const ST& shape, layout_type);
 
         template <class ST>
         const_stepper stepper_begin(const ST& shape) const;
         template <class ST>
-        const_stepper stepper_end(const ST& shape) const;
+        const_stepper stepper_end(const ST& shape, layout_type) const;
 
     private:
 
@@ -152,7 +151,7 @@ namespace xt
         const indices_type m_indices;
         const inner_shape_type m_shape;
 
-        void assign_temporary_impl(temporary_type& tmp);
+        void assign_temporary_impl(temporary_type&& tmp);
 
         friend class xview_semantic<xindexview<CT, I>>;
     };
@@ -182,7 +181,6 @@ namespace xt
     template <class ECT, class CCT>
     class xfiltration
     {
-
     public:
 
         using self_type = xfiltration<ECT, CCT>;
@@ -262,7 +260,7 @@ namespace xt
     }
 
     template <class CT, class I>
-    inline void xindexview<CT, I>::assign_temporary_impl(temporary_type& tmp)
+    inline void xindexview<CT, I>::assign_temporary_impl(temporary_type&& tmp)
     {
         std::copy(tmp.cbegin(), tmp.cend(), this->xbegin());
     }
@@ -367,7 +365,7 @@ namespace xt
     /**
      * Returns a reference to the element at the specified position in the xindexview.
      * @param first iterator starting the sequence of indices
-     * The number of indices in the squence should be equal to or greater 1.
+     * The number of indices in the sequence should be equal to or greater 1.
      */
     template <class CT, class I>
     template <class It>
@@ -427,7 +425,7 @@ namespace xt
 
     template <class CT, class I>
     template <class ST>
-    inline auto xindexview<CT, I>::stepper_end(const ST& shape) -> stepper
+    inline auto xindexview<CT, I>::stepper_end(const ST& shape, layout_type) -> stepper
     {
         size_type offset = shape.size() - dimension();
         return stepper(this, offset, true);
@@ -443,7 +441,7 @@ namespace xt
 
     template <class CT, class I>
     template <class ST>
-    inline auto xindexview<CT, I>::stepper_end(const ST& shape) const -> const_stepper
+    inline auto xindexview<CT, I>::stepper_end(const ST& shape, layout_type) const -> const_stepper
     {
         size_type offset = shape.size() - dimension();
         return const_stepper(this, offset, true);
