@@ -49,28 +49,6 @@ namespace xt
             }
         };
 
-        template <class T>
-        struct minimum
-        {
-            using result_type = T;
-
-            constexpr result_type operator()(const T& t1, const T& t2) const noexcept
-            {
-                return (t1 < t2) ? t1 : t2;
-            }
-        };
-
-        template <class T>
-        struct maximum
-        {
-            using result_type = T;
-
-            constexpr result_type operator()(const T& t1, const T& t2) const noexcept
-            {
-                return (t1 > t2) ? t1 : t2;
-            }
-        };
-
         template <template <class...> class F, class... E>
         inline auto make_xfunction(E&&... e) noexcept
         {
@@ -428,120 +406,6 @@ namespace xt
         return detail::make_xfunction<detail::conditional_ternary>(std::forward<E1>(e1), std::forward<E2>(e2), std::forward<E3>(e3));
     }
 
-    /**
-    * @ingroup logical_operators
-    * @brief Elementwise maximum
-    *
-    * Returns an \ref xfunction for the element-wise
-    * maximum between e1 and e2.
-    * @param e1 an \ref xexpression
-    * @param e2 an \ref xexpression
-    * @return an \ref xfunction
-    */
-    template <class E1, class E2>
-    inline auto maximum(E1&& e1, E2&& e2) noexcept
-        -> detail::xfunction_type_t<detail::maximum, E1, E2>
-    {
-        return detail::make_xfunction<detail::maximum>(std::forward<E1>(e1), std::forward<E2>(e2));
-    }
-
-    /**
-    * @ingroup logical_operators
-    * @brief Elementwise minimum
-    *
-    * Returns an \ref xfunction for the element-wise
-    * minimum between e1 and e2.
-    * @param e1 an \ref xexpression
-    * @param e2 an \ref xexpression
-    * @return an \ref xfunction
-    */
-    template <class E1, class E2>
-    inline auto minimum(E1&& e1, E2&& e2) noexcept
-        -> detail::xfunction_type_t<detail::minimum, E1, E2>
-    {
-        return detail::make_xfunction<detail::minimum>(std::forward<E1>(e1), std::forward<E2>(e2));
-    }
-
-    /**
-     * @ingroup logical_operators
-     * @brief Maximum element along given axis.
-     *
-     * Returns an \ref xreducer for the maximum of elements over given
-     * \em axes.
-     * @param e an \ref xexpression
-     * @param axes the axes along which the maximum is found (optional)
-     * @return an \ref xreducer
-     */
-    template <class E, class X>
-    inline auto amax(E&& e, X&& axes) noexcept
-    {
-        using functor_type = detail::maximum<typename std::decay_t<E>::value_type>;
-        return reduce(functor_type(), std::forward<E>(e), std::forward<X>(axes));
-    }
-
-    template <class E>
-    inline auto amax(E&& e) noexcept
-    {
-        using functor_type = detail::maximum<typename std::decay_t<E>::value_type>;
-        return reduce(functor_type(), std::forward<E>(e));
-    }
-
-#ifdef X_OLD_CLANG
-    template <class E, class I>
-    inline auto amax(E&& e, std::initializer_list<I> axes) noexcept
-    {
-        using functor_type = detail::maximum<typename std::decay_t<E>::value_type>;
-        return reduce(functor_type(), std::forward<E>(e), axes);
-    }
-#else
-    template <class E, class I, std::size_t N>
-    inline auto amax(E&& e, const I (&axes)[N]) noexcept
-    {
-        using functor_type = detail::maximum<typename std::decay_t<E>::value_type>;
-        return reduce(functor_type(), std::forward<E>(e), axes);
-    }
-#endif
-
-    /**
-     * @ingroup logical_operators
-     * @brief Minimum element along given axis.
-     *
-     * Returns an \ref xreducer for the minimum of elements over given
-     * \em axes.
-     * @param e an \ref xexpression
-     * @param axes the axes along which the minimum is found (optional)
-     * @return an \ref xreducer
-     */
-    template <class E, class X>
-    inline auto amin(E&& e, X&& axes) noexcept
-    {
-        using functor_type = detail::minimum<typename std::decay_t<E>::value_type>;
-        return reduce(functor_type(), std::forward<E>(e), std::forward<X>(axes));
-    }
-
-    template <class E>
-    inline auto amin(E&& e) noexcept
-    {
-        using functor_type = detail::minimum<typename std::decay_t<E>::value_type>;
-        return reduce(functor_type(), std::forward<E>(e));
-    }
-
-#ifdef X_OLD_CLANG
-    template <class E, class I>
-    inline auto amin(E&& e, std::initializer_list<I> axes) noexcept
-    {
-        using functor_type = detail::minimum<typename std::decay_t<E>::value_type>;
-        return reduce(functor_type(), std::forward<E>(e), axes);
-    }
-#else
-    template <class E, class I, std::size_t N>
-    inline auto amin(E&& e, const I (&axes)[N]) noexcept
-    {
-        using functor_type = detail::minimum<typename std::decay_t<E>::value_type>;
-        return reduce(functor_type(), std::forward<E>(e), axes);
-    }
-#endif
-
     /**
      * @ingroup logical_operators
      * @brief return vector of indices where T is not zero
@@ -557,7 +421,7 @@ namespace xt
         using index_type = xindex_type_t<typename T::shape_type>;
         using size_type = typename T::size_type;
 
-        index_type idx(arr.dimension(), 0);
+        auto idx = make_sequence<index_type>(arr.dimension(), 0);
         std::vector<index_type> indices;
 
         auto next_idx = [&shape](index_type& idx)
@@ -581,7 +445,7 @@ namespace xt
         size_type total_size = compute_size(shape);
         for (size_type i = 0; i < total_size; i++, next_idx(idx))
         {
-            if (arr[idx])
+            if (arr.element(std::begin(idx), std::end(idx)))
             {
                 indices.push_back(idx);
             }
