@@ -1,5 +1,5 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille and Sylvain Corlay                     *
+* Copyright (c) 2016, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -44,15 +44,14 @@ namespace xt
         using const_iterator = const value_type*;
         using const_stepper = xscalar_stepper<true, CT>;
         using stepper = xscalar_stepper<false, CT>;
-        using const_broadcast_iterator = xiterator<const_stepper, inner_shape_type*>;
-        using broadcast_iterator = xiterator<stepper, inner_shape_type*>;
+        using reverse_iterator = std::reverse_iterator<iterator>;
+        using const_reverse_iterator = std::reverse_iterator<const_iterator>;
     };
 
     template <class CT>
     class xscalar : public xexpression<xscalar<CT>>,
                     public xiterable<xscalar<CT>>
     {
-
     public:
 
         using self_type = xscalar<CT>;
@@ -72,9 +71,6 @@ namespace xt
         using stepper = typename iterable_base::stepper;
         using const_stepper = typename iterable_base::const_stepper;
 
-        using broadcast_iterator = typename iterable_base::broadcast_iterator;
-        using const_broadcast_iterator = typename iterable_base::const_broadcast_iterator;
-
         using iterator = typename iterable_base::iterator;
         using const_iterator = typename iterable_base::const_iterator;
 
@@ -124,12 +120,12 @@ namespace xt
         template <class S>
         stepper stepper_begin(const S& shape) noexcept;
         template <class S>
-        stepper stepper_end(const S& shape) noexcept;
+        stepper stepper_end(const S& shape, layout_type l) noexcept;
 
         template <class S>
         const_stepper stepper_begin(const S& shape) const noexcept;
         template <class S>
-        const_stepper stepper_end(const S& shape) const noexcept;
+        const_stepper stepper_end(const S& shape, layout_type l) const noexcept;
 
         dummy_iterator dummy_begin() noexcept;
         dummy_iterator dummy_end() noexcept;
@@ -137,6 +133,9 @@ namespace xt
         const_dummy_iterator dummy_begin() const noexcept;
         const_dummy_iterator dummy_end() const noexcept;
 
+        reference data_element(size_type i) noexcept;
+        const_reference data_element(size_type i) const noexcept;
+
     private:
 
         CT m_value;
@@ -155,7 +154,6 @@ namespace xt
     template <bool is_const, class CT>
     class xscalar_stepper
     {
-
     public:
 
         using self_type = xscalar_stepper<is_const, CT>;
@@ -180,8 +178,10 @@ namespace xt
         void step(size_type dim, size_type n = 1) noexcept;
         void step_back(size_type dim, size_type n = 1) noexcept;
         void reset(size_type dim) noexcept;
+        void reset_back(size_type dim) noexcept;
 
-        void to_end() noexcept;
+        void to_begin() noexcept;
+        void to_end(layout_type l) noexcept;
 
         bool equal(const self_type& rhs) const noexcept;
 
@@ -198,14 +198,13 @@ namespace xt
     bool operator!=(const xscalar_stepper<is_const, CT>& lhs,
                     const xscalar_stepper<is_const, CT>& rhs) noexcept;
 
-    /********************
+    /*******************
      * xdummy_iterator *
-     ********************/
+     *******************/
 
     template <bool is_const, class CT>
     class xdummy_iterator
     {
-
     public:
 
         using self_type = xdummy_iterator<is_const, CT>;
@@ -422,7 +421,7 @@ namespace xt
 
     template <class CT>
     template <class S>
-    inline auto xscalar<CT>::stepper_end(const S&) noexcept -> stepper
+    inline auto xscalar<CT>::stepper_end(const S&, layout_type) noexcept -> stepper
     {
         return stepper(this + 1);
     }
@@ -436,7 +435,7 @@ namespace xt
 
     template <class CT>
     template <class S>
-    inline auto xscalar<CT>::stepper_end(const S&) const noexcept -> const_stepper
+    inline auto xscalar<CT>::stepper_end(const S&, layout_type) const noexcept -> const_stepper
     {
         return const_stepper(this + 1);
     }
@@ -465,6 +464,18 @@ namespace xt
         return const_dummy_iterator(this);
     }
 
+    template <class CT>
+    inline auto xscalar<CT>::data_element(size_type) noexcept->reference
+    {
+        return m_value;
+    }
+    
+    template <class CT>
+    inline auto xscalar<CT>::data_element(size_type) const noexcept->const_reference
+    {
+        return m_value;
+    }
+
     template <class T>
     inline xscalar<T&> xref(T& t)
     {
@@ -509,9 +520,20 @@ namespace xt
     }
 
     template <bool is_const, class CT>
-    inline void xscalar_stepper<is_const, CT>::to_end() noexcept
+    inline void xscalar_stepper<is_const, CT>::reset_back(size_type /*dim*/) noexcept
     {
-        p_c = p_c->stepper_end(p_c->shape()).p_c;
+    }
+
+    template <bool is_const, class CT>
+    inline void xscalar_stepper<is_const, CT>::to_begin() noexcept
+    {
+        p_c = p_c->stepper_begin(p_c->shap()).pc;
+    }
+
+    template <bool is_const, class CT>
+    inline void xscalar_stepper<is_const, CT>::to_end(layout_type l) noexcept
+    {
+        p_c = p_c->stepper_end(p_c->shape(), l).p_c;
     }
 
     template <bool is_const, class CT>
@@ -534,9 +556,9 @@ namespace xt
         return !(lhs.equal(rhs));
     }
 
-    /***********************************
+    /**********************************
      * xdummy_iterator implementation *
-     ***********************************/
+     **********************************/
 
     template <bool is_const, class CT>
     inline xdummy_iterator<is_const, CT>::xdummy_iterator(container_type* c) noexcept
