@@ -33,46 +33,46 @@ static std::string big_exp = R"xio({{ 4.259106e+000, -1.430581e+000,  3.817697e+
  {-1.194856e+000, -8.325470e+000,  4.246604e+000, -1.442730e+000}})xio";
 
 
-static std::string complex_numbers = R"xio({{ 4.259106-4.705763j, -1.430581+0.520437j,  3.817697-2.597525j, 
-   4.383006+3.84805j , -0.177621-0.397616j,  5.600555-3.068297j, 
-  -1.781513-2.063075j,  1.593886+3.179275j, -7.200985+0.594874j, 
-  -1.979649+1.779254j},
- { 2.54634 +3.091267j, -3.516982+3.685722j, -5.104814-0.820075j, 
-   3.895104-4.410618j,  1.878048-0.21541j ,  2.63584 +0.211594j, 
-  -1.194856+0.80632j , -8.32547 -1.90801j ,  4.246604+4.198826j, 
-  -1.44273 +1.553475j},
- {-4.04439 -1.507586j, -0.158305+0.41094j ,  4.805928-0.509466j, 
-  -2.845422-2.17679j , -1.655801-2.041235j,  3.094426+0.63481j , 
-  -2.523971+2.151913j, -5.309743+0.176187j,  9.759906-1.479582j, 
-   5.319919+1.327789j},
- { 5.540089+3.73119j , -9.440361-3.321116j, -6.52187 +4.874783j, 
-  -6.918355-1.505851j, -8.45827 +3.262051j,  7.797314-4.345385j, 
-   5.007574-4.946467j,  3.868065+3.869446j,  0.235268+4.113181j, 
-  -0.714639-3.005922j},
- { 1.368614-4.724486j, -3.949011+3.173349j, -0.053824-2.581724j, 
-   3.665258-4.43917j ,  8.333974-4.324392j, -7.821421-0.261281j, 
-  -0.090164+0.094293j, -5.343281-0.832049j, -1.262787-2.441571j, 
-   5.03086 -1.184879j},
- {-0.382158+3.073844j,  5.954568+3.110391j, -4.345941+1.861869j, 
-  -1.331635+4.926136j, -9.804853-0.948148j, -3.18408 -3.098263j, 
-   3.78544 -0.118255j,  7.387386+0.023696j, -4.643924-0.966452j, 
-  -0.865042-3.189046j},
- {-4.634374-4.519702j,  6.741056+1.046913j, -4.589707-2.340888j, 
-   0.60124 +3.81347j , -6.492547+0.840461j, -3.700671-0.225395j, 
-   7.822182-3.28485j , -6.393274-4.4338j  , -0.113676-3.429648j, 
-  -5.754038-1.613825j},
- { 0.417537-0.59357j , -6.798001+1.838431j,  8.381136-2.941283j, 
-   9.885798+1.164337j, -0.459049+4.244716j, -3.80233 -4.203717j, 
-   0.144086-0.146032j, -4.384136+1.985636j,  5.276734-4.184315j, 
-  -7.829154+3.691024j},
- { 0.233094-0.668943j,  8.195388-0.056699j, -5.63248 -0.042595j, 
-  -2.737919-1.292779j,  7.099456+4.221287j,  4.227836+3.412782j, 
-  -2.141112+1.555259j, -5.37397 -0.37315j , -2.396506+0.650113j, 
-   0.983242-0.760845j},
- { 1.134381-2.117881j, -9.917307-2.360193j,  2.76045 +1.907382j, 
-  -8.84704 +3.328365j, -9.139462+0.923916j,  7.501023-0.334774j, 
-  -4.148248+1.7366j  ,  5.255353-1.085206j, -2.642695+2.021044j, 
-   7.470045+4.656976j}})xio";
+static std::string complex_numbers = R"xio({{ 4.259106-4.705763i, -1.430581+0.520437i,  3.817697-2.597525i, 
+   4.383006+3.84805i , -0.177621-0.397616i,  5.600555-3.068297i, 
+  -1.781513-2.063075i,  1.593886+3.179275i, -7.200985+0.594874i, 
+  -1.979649+1.779254i},
+ { 2.54634 +3.091267i, -3.516982+3.685722i, -5.104814-0.820075i, 
+   3.895104-4.410618i,  1.878048-0.21541i ,  2.63584 +0.211594i, 
+  -1.194856+0.80632i , -8.32547 -1.90801i ,  4.246604+4.198826i, 
+  -1.44273 +1.553475i},
+ {-4.04439 -1.507586i, -0.158305+0.41094i ,  4.805928-0.509466i, 
+  -2.845422-2.17679i , -1.655801-2.041235i,  3.094426+0.63481i , 
+  -2.523971+2.151913i, -5.309743+0.176187i,  9.759906-1.479582i, 
+   5.319919+1.327789i},
+ { 5.540089+3.73119i , -9.440361-3.321116i, -6.52187 +4.874783i, 
+  -6.918355-1.505851i, -8.45827 +3.262051i,  7.797314-4.345385i, 
+   5.007574-4.946467i,  3.868065+3.869446i,  0.235268+4.113181i, 
+  -0.714639-3.005922i},
+ { 1.368614-4.724486i, -3.949011+3.173349i, -0.053824-2.581724i, 
+   3.665258-4.43917i ,  8.333974-4.324392i, -7.821421-0.261281i, 
+  -0.090164+0.094293i, -5.343281-0.832049i, -1.262787-2.441571i, 
+   5.03086 -1.184879i},
+ {-0.382158+3.073844i,  5.954568+3.110391i, -4.345941+1.861869i, 
+  -1.331635+4.926136i, -9.804853-0.948148i, -3.18408 -3.098263i, 
+   3.78544 -0.118255i,  7.387386+0.023696i, -4.643924-0.966452i, 
+  -0.865042-3.189046i},
+ {-4.634374-4.519702i,  6.741056+1.046913i, -4.589707-2.340888i, 
+   0.60124 +3.81347i , -6.492547+0.840461i, -3.700671-0.225395i, 
+   7.822182-3.28485i , -6.393274-4.4338i  , -0.113676-3.429648i, 
+  -5.754038-1.613825i},
+ { 0.417537-0.59357i , -6.798001+1.838431i,  8.381136-2.941283i, 
+   9.885798+1.164337i, -0.459049+4.244716i, -3.80233 -4.203717i, 
+   0.144086-0.146032i, -4.384136+1.985636i,  5.276734-4.184315i, 
+  -7.829154+3.691024i},
+ { 0.233094-0.668943i,  8.195388-0.056699i, -5.63248 -0.042595i, 
+  -2.737919-1.292779i,  7.099456+4.221287i,  4.227836+3.412782i, 
+  -2.141112+1.555259i, -5.37397 -0.37315i , -2.396506+0.650113i, 
+   0.983242-0.760845i},
+ { 1.134381-2.117881i, -9.917307-2.360193i,  2.76045 +1.907382i, 
+  -8.84704 +3.328365i, -9.139462+0.923916i,  7.501023-0.334774i, 
+  -4.148248+1.7366i  ,  5.255353-1.085206i, -2.642695+2.021044i, 
+   7.470045+4.656976i}})xio";
 
 
 static std::string cut_long = R"xio({{1, 1, 1, ..., 1, 1, 1},
@@ -527,45 +527,45 @@ static std::string custom_formatter_result = R"xio({{     0x1,      0x2,      0x
  {    0x64,     0xc8,    0x3e8, 0x989680}})xio";
 
 
-static std::string complex_zero_erasing = R"xio({{ 0.212955-0.470576j, -0.071529+0.052044j,  0.190885-0.259752j, 
-   0.21915 +0.384805j, -0.008881-0.039762j,  0.280028-0.30683j , 
-  -0.089076-0.206307j,  0.079694+0.317928j, -0.360049+0.059487j, 
-  -0.098982+0.177925j},
- { 0.127317+0.309127j, -0.175849+0.368572j, -0.255241-0.082008j, 
-   0.194755-0.441062j,  0.093902-0.021541j,  0.131792+0.021159j, 
-  -0.059743+0.080632j, -0.416274-0.190801j,  0.21233 +0.419883j, 
-  -0.072137+0.155348j},
- {-0.20222 -0.150759j, -0.007915+0.041094j,  0.240296-0.050947j, 
-  -0.142271-0.217679j, -0.08279 -0.204123j,  0.154721+0.063481j, 
-  -0.126199+0.215191j, -0.265487+0.017619j,  0.487995-0.147958j, 
-   0.265996+0.132779j},
- { 0.277004+0.373119j, -0.472018-0.332112j, -0.326093+0.487478j, 
-  -0.345918-0.150585j, -0.422914+0.326205j,  0.389866-0.434538j, 
-   0.250379-0.494647j,  0.193403+0.386945j,  0.011763+0.411318j, 
-  -0.035732-0.300592j},
- { 0.068431-0.472449j, -0.197451+0.317335j, -0.002691-0.258172j, 
-   0.183263-0.443917j,  0.416699-0.432439j, -0.391071-0.026128j, 
-  -0.004508+0.009429j, -0.267164-0.083205j, -0.063139-0.244157j, 
-   0.251543-0.118488j},
- {-0.019108+0.307384j,  0.297728+0.311039j, -0.217297+0.186187j, 
-  -0.066582+0.492614j, -0.490243-0.094815j, -0.159204-0.309826j, 
-   0.189272-0.011826j,  0.369369+0.00237j , -0.232196-0.096645j, 
-  -0.043252-0.318905j},
- {-0.231719-0.45197j ,  0.337053+0.104691j, -0.229485-0.234089j, 
-   0.030062+0.381347j, -0.324627+0.084046j, -0.185034-0.02254j , 
-   0.391109-0.328485j, -0.319664-0.44338j , -0.005684-0.342965j, 
-  -0.287702-0.161383j},
- { 0.020877-0.059357j, -0.3399  +0.183843j,  0.419057-0.294128j, 
-   0.49429 +0.116434j, -0.022952+0.424472j, -0.190116-0.420372j, 
-   0.007204-0.014603j, -0.219207+0.198564j,  0.263837-0.418432j, 
-  -0.391458+0.369102j},
- { 0.011655-0.066894j,  0.409769-0.00567j , -0.281624-0.00426j , 
-  -0.136896-0.129278j,  0.354973+0.422129j,  0.211392+0.341278j, 
-  -0.107056+0.155526j, -0.268699-0.037315j, -0.119825+0.065011j, 
-   0.049162-0.076084j},
- { 0.056719-0.211788j, -0.495865-0.236019j,  0.138023+0.190738j, 
-  -0.442352+0.332837j, -0.456973+0.092392j,  0.375051-0.033477j, 
-  -0.207412+0.17366j ,  0.262768-0.108521j, -0.132135+0.202104j, 
-   0.373502+0.465698j}})xio";
+static std::string complex_zero_erasing = R"xio({{ 0.212955-0.470576i, -0.071529+0.052044i,  0.190885-0.259752i, 
+   0.21915 +0.384805i, -0.008881-0.039762i,  0.280028-0.30683i , 
+  -0.089076-0.206307i,  0.079694+0.317928i, -0.360049+0.059487i, 
+  -0.098982+0.177925i},
+ { 0.127317+0.309127i, -0.175849+0.368572i, -0.255241-0.082008i, 
+   0.194755-0.441062i,  0.093902-0.021541i,  0.131792+0.021159i, 
+  -0.059743+0.080632i, -0.416274-0.190801i,  0.21233 +0.419883i, 
+  -0.072137+0.155348i},
+ {-0.20222 -0.150759i, -0.007915+0.041094i,  0.240296-0.050947i, 
+  -0.142271-0.217679i, -0.08279 -0.204123i,  0.154721+0.063481i, 
+  -0.126199+0.215191i, -0.265487+0.017619i,  0.487995-0.147958i, 
+   0.265996+0.132779i},
+ { 0.277004+0.373119i, -0.472018-0.332112i, -0.326093+0.487478i, 
+  -0.345918-0.150585i, -0.422914+0.326205i,  0.389866-0.434538i, 
+   0.250379-0.494647i,  0.193403+0.386945i,  0.011763+0.411318i, 
+  -0.035732-0.300592i},
+ { 0.068431-0.472449i, -0.197451+0.317335i, -0.002691-0.258172i, 
+   0.183263-0.443917i,  0.416699-0.432439i, -0.391071-0.026128i, 
+  -0.004508+0.009429i, -0.267164-0.083205i, -0.063139-0.244157i, 
+   0.251543-0.118488i},
+ {-0.019108+0.307384i,  0.297728+0.311039i, -0.217297+0.186187i, 
+  -0.066582+0.492614i, -0.490243-0.094815i, -0.159204-0.309826i, 
+   0.189272-0.011826i,  0.369369+0.00237i , -0.232196-0.096645i, 
+  -0.043252-0.318905i},
+ {-0.231719-0.45197i ,  0.337053+0.104691i, -0.229485-0.234089i, 
+   0.030062+0.381347i, -0.324627+0.084046i, -0.185034-0.02254i , 
+   0.391109-0.328485i, -0.319664-0.44338i , -0.005684-0.342965i, 
+  -0.287702-0.161383i},
+ { 0.020877-0.059357i, -0.3399  +0.183843i,  0.419057-0.294128i, 
+   0.49429 +0.116434i, -0.022952+0.424472i, -0.190116-0.420372i, 
+   0.007204-0.014603i, -0.219207+0.198564i,  0.263837-0.418432i, 
+  -0.391458+0.369102i},
+ { 0.011655-0.066894i,  0.409769-0.00567i , -0.281624-0.00426i , 
+  -0.136896-0.129278i,  0.354973+0.422129i,  0.211392+0.341278i, 
+  -0.107056+0.155526i, -0.268699-0.037315i, -0.119825+0.065011i, 
+   0.049162-0.076084i},
+ { 0.056719-0.211788i, -0.495865-0.236019i,  0.138023+0.190738i, 
+  -0.442352+0.332837i, -0.456973+0.092392i,  0.375051-0.033477i, 
+  -0.207412+0.17366i ,  0.262768-0.108521i, -0.132135+0.202104i, 
+   0.373502+0.465698i}})xio";
 
 
