@@ -338,7 +338,7 @@ namespace xt
             vec.reshape(rm.m_shape, layout_type::row_major);
             assign_array(vec, rm.m_assigner);
             EXPECT_EQ(vec.data(), rm.m_data);
-            EXPECT_EQ(vec(2, 1, 0), vec(2, 1));
+            EXPECT_EQ(vec(0, 1, 1), vec(1, 1));
             EXPECT_EQ(vec(2, 1, 3), vec(2, 2, 2, 1, 3));
             test_bound_check(vec);
         }
@@ -349,7 +349,7 @@ namespace xt
             vec.reshape(cm.m_shape, layout_type::column_major);
             assign_array(vec, cm.m_assigner);
             EXPECT_EQ(vec.data(), cm.m_data);
-            EXPECT_EQ(vec(2, 1, 0), vec(2, 1));
+            EXPECT_EQ(vec(0, 1, 1), vec(1, 1));
             EXPECT_EQ(vec(2, 1, 3), vec(2, 2, 2, 1, 3));
             test_bound_check(vec);
         }
@@ -360,7 +360,7 @@ namespace xt
             vec.reshape(cem.m_shape, cem.m_strides);
             assign_array(vec, cem.m_assigner);
             EXPECT_EQ(vec.data(), cem.m_data);
-            EXPECT_EQ(vec(2, 1, 0), vec(2, 1));
+            EXPECT_EQ(vec(0, 1, 1), vec(1, 1));
             EXPECT_EQ(vec(2, 1, 3), vec(2, 2, 2, 1, 3));
             test_bound_check(vec);
         }
@@ -371,12 +371,76 @@ namespace xt
             vec.reshape(usr.m_shape, layout_type::row_major);
             assign_array(vec, usr.m_assigner);
             EXPECT_EQ(vec.data(), usr.m_data);
-            EXPECT_EQ(vec(2, 0, 0), vec(2, 0));
+            EXPECT_EQ(vec(0, 1, 0), vec(1, 0));
             EXPECT_EQ(vec(2, 0, 3), vec(2, 2, 2, 0, 3));
             test_bound_check(vec);
         }
     }
 
+    template <class V, class C = std::vector<std::size_t>>
+    void test_element(V& vec)
+    {
+        {
+            SCOPED_TRACE("row_major access");
+            row_major_result<C> rm;
+            vec.reshape(rm.m_shape, layout_type::row_major);
+            assign_array(vec, rm.m_assigner);
+            EXPECT_EQ(vec.data(), rm.m_data);
+            std::vector<std::size_t> index1 = {0, 1, 1};
+            std::vector<std::size_t> index2 = {1, 1};
+            std::vector<std::size_t> index3 = {2, 1, 3};
+            std::vector<std::size_t> index4 = {2, 2, 2, 1, 3};
+            EXPECT_EQ(vec.element(index1.begin(), index1.end()), vec.element(index2.begin(), index2.end()));
+            EXPECT_EQ(vec.element(index3.begin(), index3.end()), vec.element(index4.begin(), index4.end()));
+            test_bound_check(vec);
+        }
+
+        {
+            SCOPED_TRACE("column_major access");
+            column_major_result<C> cm;
+            vec.reshape(cm.m_shape, layout_type::column_major);
+            assign_array(vec, cm.m_assigner);
+            EXPECT_EQ(vec.data(), cm.m_data);
+            std::vector<std::size_t> index1 = {0, 1, 1};
+            std::vector<std::size_t> index2 = {1, 1};
+            std::vector<std::size_t> index3 = {2, 1, 3};
+            std::vector<std::size_t> index4 = {2, 2, 2, 1, 3};
+            EXPECT_EQ(vec.element(index1.begin(), index1.end()), vec.element(index2.begin(), index2.end()));
+            EXPECT_EQ(vec.element(index3.begin(), index3.end()), vec.element(index4.begin(), index4.end()));
+            test_bound_check(vec);
+        }
+
+        {
+            SCOPED_TRACE("central_major access");
+            central_major_result<C> cem;
+            vec.reshape(cem.m_shape, cem.m_strides);
+            assign_array(vec, cem.m_assigner);
+            EXPECT_EQ(vec.data(), cem.m_data);
+            std::vector<std::size_t> index1 = {0, 1, 1};
+            std::vector<std::size_t> index2 = {1, 1};
+            std::vector<std::size_t> index3 = {2, 1, 3};
+            std::vector<std::size_t> index4 = {2, 2, 2, 1, 3};
+            EXPECT_EQ(vec.element(index1.begin(), index1.end()), vec.element(index2.begin(), index2.end()));
+            EXPECT_EQ(vec.element(index3.begin(), index3.end()), vec.element(index4.begin(), index4.end()));
+            test_bound_check(vec);
+        }
+
+        {
+            SCOPED_TRACE("unit_shape access");
+            unit_shape_result<C> usr;
+            vec.reshape(usr.m_shape, layout_type::row_major);
+            assign_array(vec, usr.m_assigner);
+            EXPECT_EQ(vec.data(), usr.m_data);
+            std::vector<std::size_t> index1 = {0, 1, 0};
+            std::vector<std::size_t> index2 = {1, 0};
+            std::vector<std::size_t> index3 = {2, 0, 3};
+            std::vector<std::size_t> index4 = {2, 2, 2, 0, 3};
+            EXPECT_EQ(vec.element(index1.begin(), index1.end()), vec.element(index2.begin(), index2.end()));
+            EXPECT_EQ(vec.element(index3.begin(), index3.end()), vec.element(index4.begin(), index4.end()));
+            test_bound_check(vec);
+        }
+    }
+
     template <class V1, class V2>
     void indexed_assign_array(V1& dst, const V2& src)
     {
@@ -399,7 +463,7 @@ namespace xt
     template <class V, class C = std::vector<std::size_t>>
     void test_indexed_access(V& vec)
     {
-        xindex index1 = {2, 1};
+        xindex index1 = {1, 1};
         xindex index2 = {2, 2, 2, 1, 3};
         {
             SCOPED_TRACE("row_major access");
@@ -407,7 +471,7 @@ namespace xt
             vec.reshape(rm.m_shape, layout_type::row_major);
             indexed_assign_array(vec, rm.m_assigner);
             EXPECT_EQ(vec.data(), rm.m_data);
-            EXPECT_EQ(vec(2, 1, 0), vec[index1]);
+            EXPECT_EQ(vec(0, 1, 1), vec[index1]);
             EXPECT_EQ(vec(2, 1, 3), vec[index2]);
         }
 
@@ -417,7 +481,7 @@ namespace xt
             vec.reshape(cm.m_shape, layout_type::column_major);
             indexed_assign_array(vec, cm.m_assigner);
             EXPECT_EQ(vec.data(), cm.m_data);
-            EXPECT_EQ(vec(2, 1, 0), vec[index1]);
+            EXPECT_EQ(vec(0, 1, 1), vec[index1]);
             EXPECT_EQ(vec(2, 1, 3), vec[index2]);
         }
 
@@ -427,7 +491,7 @@ namespace xt
             vec.reshape(cem.m_shape, cem.m_strides);
             indexed_assign_array(vec, cem.m_assigner);
             EXPECT_EQ(vec.data(), cem.m_data);
-            EXPECT_EQ(vec(2, 1, 0), vec[index1]);
+            EXPECT_EQ(vec(0, 1, 1), vec[index1]);
             EXPECT_EQ(vec(2, 1, 3), vec[index2]);
         }
 
@@ -437,9 +501,9 @@ namespace xt
             vec.reshape(usr.m_shape, layout_type::row_major);
             indexed_assign_array(vec, usr.m_assigner);
             EXPECT_EQ(vec.data(), usr.m_data);
-            xindex id1 = {2, 0};
+            xindex id1 = {1, 0};
             xindex id2 = {2, 2, 2, 0, 3};
-            EXPECT_EQ(vec(2, 0, 0), vec[id1]);
+            EXPECT_EQ(vec(0, 1, 0), vec[id1]);
             EXPECT_EQ(vec(2, 0, 3), vec[id2]);
         }
     }
