@@ -24,7 +24,7 @@ namespace xt
         ASSERT_EQ(5.0, m1_broadcast(0, 1, 1));
         ASSERT_EQ(m1_broadcast.layout(), m1.layout());
 
-        auto shape = std::vector<std::size_t> {1, 2, 3};
+        auto shape = std::vector<std::size_t>{1, 2, 3};
         auto m1_broadcast2 = broadcast(m1, shape);
         ASSERT_EQ(1.0, m1_broadcast2(0, 0, 0));
         ASSERT_EQ(4.0, m1_broadcast2(0, 1, 0));
@@ -59,58 +59,67 @@ namespace xt
 
     TEST(xbroadcast, xiterator)
     {
-        xarray<int> m1 = { 1, 2, 3 };
-        auto m1_broadcast = broadcast(m1, { 2, 3 });
+        xarray<int> m1 = {1, 2, 3};
+        auto m1_broadcast = broadcast(m1, {2, 3});
         size_t nb_iter = 3;
+        using shape_type = std::vector<size_t>;
 
         // broadcast_iterator
         {
-            auto iter = m1_broadcast.xbegin();
-            auto iter_end = m1_broadcast.xend();
-            for (size_t i = 0; i < nb_iter; ++i) ++iter;
+            auto iter = m1_broadcast.template xbegin<layout_type::row_major>();
+            auto iter_end = m1_broadcast.template xend<layout_type::row_major>();
+            for (size_t i = 0; i < nb_iter; ++i)
+                ++iter;
             EXPECT_EQ(1, *iter);
-            for (size_t i = 0; i < nb_iter; ++i) ++iter;
+            for (size_t i = 0; i < nb_iter; ++i)
+                ++iter;
             EXPECT_EQ(iter, iter_end);
         }
 
         // shaped_xiterator
         {
-            std::vector<size_t> shape = { 2, 2, 3 };
-            auto iter = m1_broadcast.xbegin(shape);
-            auto iter_end = m1_broadcast.xend(shape);
-            for (size_t i = 0; i < 2 * nb_iter; ++i) ++iter;
+            shape_type shape = {2, 2, 3};
+            auto iter = m1_broadcast.template xbegin<shape_type, layout_type::row_major>(shape);
+            auto iter_end = m1_broadcast.template xend<shape_type, layout_type::row_major>(shape);
+            for (size_t i = 0; i < 2 * nb_iter; ++i)
+                ++iter;
             EXPECT_EQ(1, *iter);
-            for (size_t i = 0; i < 2 * nb_iter; ++i) ++iter;
+            for (size_t i = 0; i < 2 * nb_iter; ++i)
+                ++iter;
             EXPECT_EQ(iter, iter_end);
         }
     }
-    
+
     TEST(xbroadcast, reverse_xiterator)
     {
-        xarray<int> m1 = { 1, 2, 3 };
-        auto m1_broadcast = broadcast(m1, { 2, 3 });
+        xarray<int> m1 = {1, 2, 3};
+        auto m1_broadcast = broadcast(m1, {2, 3});
         size_t nb_iter = 3;
+        using shape_type = std::vector<size_t>;
 
         // reverse_broadcast_iterator
         {
-            auto iter = m1_broadcast.xrbegin();
-            auto iter_end = m1_broadcast.xrend();
-            for (size_t i = 0; i < nb_iter; ++i) ++iter;
+            auto iter = m1_broadcast.template xrbegin<layout_type::row_major>();
+            auto iter_end = m1_broadcast.template xrend<layout_type::row_major>();
+            for (size_t i = 0; i < nb_iter; ++i)
+                ++iter;
             EXPECT_EQ(3, *iter);
-            for (size_t i = 0; i < nb_iter; ++i) ++iter;
+            for (size_t i = 0; i < nb_iter; ++i)
+                ++iter;
             EXPECT_EQ(iter, iter_end);
         }
 
         // reverse_shaped_xiterator
         {
-            std::vector<size_t> shape = { 2, 2, 3 };
-            auto iter = m1_broadcast.xrbegin(shape);
-            auto iter_end = m1_broadcast.xrend(shape);
-            for (size_t i = 0; i < 2 * nb_iter; ++i) ++iter;
+            shape_type shape = {2, 2, 3};
+            auto iter = m1_broadcast.template xrbegin<shape_type, layout_type::row_major>(shape);
+            auto iter_end = m1_broadcast.template xrend<shape_type, layout_type::row_major>(shape);
+            for (size_t i = 0; i < 2 * nb_iter; ++i)
+                ++iter;
             EXPECT_EQ(3, *iter);
-            for (size_t i = 0; i < 2 * nb_iter; ++i) ++iter;
+            for (size_t i = 0; i < 2 * nb_iter; ++i)
+                ++iter;
             EXPECT_EQ(iter, iter_end);
         }
     }
 }
-
